// select `Tools`->`Board:`->ESP32 -> ESP32 Dev Module
/* Code adapted by Brett Oliver from original code by Shinsaku Hiura
 *  Original design https://www.thingiverse.com/thing:5234371
 *  
 *  This clock http://www.brettoliver.org.uk/Card_Clock/card_clock.htm
 *  http://www.brettoliver.org.uk/
 * Brett v1 add Kenley Time Server details
 * Brett v3 adapt to ESP32WROOM
 * v4 remove reverse on start
 * v5 output pins changed to stepper modules to suit verobord layout
 * v6 adding LED outputs-removing adafruit Neopixel
 * v7 add start button - prevents clock auto restarting on power up
 * v8 add NTP fail LED
 * v9 add check NTP time every hour removes comments on 218 to 221 to check at midnight or 224 to 227 to check on the hour
 * v10 add summertime button removed hourly check off time as per v9 above Hour corrects on next minute change
 *  
 */

//#include <Adafruit_NeoPixel.h>
#include "WiFi.h"

// NTP settings
int TIMEZONE = 0; // timezone (GMT = 0, Japan = 9 1 = gmt summertime)

#define NTP_SERVER "uk.pool.ntp.org" //enter your time server here

#define WIFI_SMARTCONFIG false

#if !WIFI_SMARTCONFIG
// if you do not use smartConfifg, please specify SSID and password here
#define WIFI_SSID "WIFI SSID" // your WiFi's SSID
#define WIFI_PASS "WIFI PASSWORD" // your WiFi's password


#endif

#define WIFI_Fail_LED_PIN 4
#define WIFI_Get_LED_PIN 12
#define WIFI_OK_LED_PIN 15
#define Start_Sw 34
#define Summertime_Sw 35
int summertime = 0;
int firstrun = 0; //blocks start until start button is pressed
int start = HIGH;

//Adafruit_NeoPixel pixels(1, LED_PIN, NEO_GRB + NEO_KHZ800);

#define STEPS_PER_ROTATION 4096 // steps for a full turn of minute rotor
#define POS 14
#define MOTOR_NUM 3
#define MOTOR_PIN 4
#define MIN1 0
#define MIN10 1
#define HOUR 2

// minimum wait for a single step of stepper
#define DELAY 1

// ports used to control the stepper motor
// if your motor rotate to the opposite direction, 
// reverse the order of the port.
int port[MOTOR_NUM][MOTOR_PIN] = {

//  {16, 17, 18, 19},   // 1min digit
//  {21, 27, 22, 23},  // 10min digit
//  {26, 25, 33, 32} // hour digit

  {19, 18, 17, 16},   // 1min digit
  {23, 22, 27, 21},  // 10min digit
  {32, 33, 25, 26} // hour digit
};

// adjust initial position slightly
int initPos[MOTOR_NUM] = {50, 70, 70};

long stepToGo[MOTOR_NUM] = {0};

// sequence of stepper motor control
int seq[8][MOTOR_PIN] = {
  {  LOW, HIGH, HIGH,  LOW},
  {  LOW,  LOW, HIGH,  LOW},
  {  LOW,  LOW, HIGH, HIGH},
  {  LOW,  LOW,  LOW, HIGH},
  { HIGH,  LOW,  LOW, HIGH},
  { HIGH,  LOW,  LOW,  LOW},
  { HIGH, HIGH,  LOW,  LOW},
  {  LOW, HIGH,  LOW,  LOW}
};

void rotateSet(int num, long step) {
  stepToGo[num] += step;
}

void rotateGo(void) {
  static long phase[MOTOR_NUM] = {0};
  int dt = 20;
  int done = 0;
  int delta;

  while(!done) {
    done = 1;
    for(int m = 0; m < MOTOR_NUM; m++) {
      long step = stepToGo[m];
      if(step == 0) {
        for(int i = 0; i < MOTOR_PIN; i++) {
          digitalWrite(port[m][i], LOW);
        }
        continue;
      }
      else if(step > 0) {
        stepToGo[m]--;
        delta = 1;
      }
      else {
        stepToGo[m]++;
        delta = 7;
      }
      done = 0;
      phase[m] = (phase[m] + delta) % 8;
      for(int i = 0; i < MOTOR_PIN; i++) {
        digitalWrite(port[m][i], seq[phase[m]][i]);
      }
    }
    delay(dt);
    if(dt > DELAY) {
      dt--; // accelerate
    }
  }
  // power cut
  for(int m = 0; m < MOTOR_NUM; m++) {
    for(int i = 0; i < MOTOR_PIN; i++) {
      digitalWrite(port[m][i], LOW);
    }
  }
}

// order of playing cards
int order[MOTOR_NUM][POS] = {
  {0, 10, 5, 8, 3, 6, 2, 13, 9, 1, 11, 4, 12, 7}, // 1min digit
 {0, 13, 7, 10, 8, 12, 6, 3, 11, 5, 1, 4, 2, 9}, // 10min digit
  {0, 2, 7, 10, 4, 12, 5, 8, 3, 11, 1, 6, 9, 13}  // hour digit

  
  
};

int currentPos[MOTOR_NUM] = {0};

void setNum(int m, int n) {
  int i, pos, newPos, advance;

  for(i = 0; i < POS; i++) {
    if(order[m][i] == n)
      newPos = i;
  }
  pos = (newPos * STEPS_PER_ROTATION) / POS;
  advance = (pos - currentPos[m] + STEPS_PER_ROTATION) % STEPS_PER_ROTATION; // always CW rotation
  rotateSet(m, advance);
  currentPos[m] = pos;
}

void setup() {
  Serial.begin(115200);
  Serial.println("start");
  pinMode(WIFI_Fail_LED_PIN, OUTPUT);
  pinMode(WIFI_Get_LED_PIN, OUTPUT);
  pinMode(WIFI_OK_LED_PIN, OUTPUT);
  pinMode(Start_Sw, INPUT);
  pinMode(Summertime_Sw, INPUT);
  
  
  digitalWrite(WIFI_Fail_LED_PIN, HIGH);  //Light NTP fail LED at start
  
  for(int m = 0; m < MOTOR_NUM; m++) {
    for(int i = 0; i < MOTOR_PIN; i++) {
      pinMode(port[m][i], OUTPUT);      
    }
  }
  //pixels.setPixelColor(0, pixels.Color(0, 255, 0)); // green
  //pixels.show();
 /* Comment out to stop autostart
  for(int m = 0; m < MOTOR_NUM; m++) {
    rotateSet(m, -STEPS_PER_ROTATION); // reverse rotation to endstop  
  }
  rotateGo();
  for(int m = 0; m < MOTOR_NUM; m++) {
    rotateSet(m, STEPS_PER_ROTATION / 2 + initPos[m]); // show 0
  }
  rotateGo();
 */ 
  //pixels.setPixelColor(0, pixels.Color(0, 0, 255)); // blue
 // pixels.show();

  wifiSetup();

  getNTP();

  //pixels.setPixelColor(0, pixels.Color(0, 0, 0)); // turn off
  //pixels.show();
}

void loop() {
  start = digitalRead(Start_Sw);
  if (start == LOW || firstrun == 1)
  {
    firstrun = 1;
  
  static int prevMin = -1;
  static int prevDay = -1;
  static int prevHour = -1;
  int hour;
  struct tm tmtime;
    
  getLocalTime(&tmtime);

  // reset itself at 0:00 to get accurate time
  if(prevDay > 0 && tmtime.tm_mday != prevDay) {
    getNTP();
 prevDay = tmtime.tm_mday;
  }

 // rechecks time when start button is pressed to sync winter/summer time
if( start == LOW )
 {
    getNTP();
  
  }
  
   

   
  if(prevMin == tmtime.tm_min) {
    return;
  }
  prevMin = tmtime.tm_min;
  
  hour = tmtime.tm_hour % 12;
  if(hour == 0) hour = 12;
  
  setNum(HOUR, hour);
  setNum(MIN10, tmtime.tm_min / 10);
  setNum(MIN1, tmtime.tm_min % 10);
  rotateGo();
  delay(900);
  
}
}// needs start button pressed to run
//////////////////////////////
// WiFi and NTP section
//////////////////////////////

void getNTP(void) {
  for(int i = 0; WiFi.status() != WL_CONNECTED; i++) {
    if(i > 30) {
      ESP.restart();
    }
    Serial.println("Waiting for WiFi connection..");
    delay(1000);
  }
// check for sumertime
summertime = digitalRead(Summertime_Sw);

if( summertime == HIGH )  // Summertime sw is off
{
  TIMEZONE = 0; //Set for GMT
}

else if( summertime == LOW )  // Summertime sw is on
{
  TIMEZONE = 1; //Set for GMT + 1
}

  configTime(TIMEZONE * 3600L, 0, NTP_SERVER);
  printLocalTime();
}

void printLocalTime() {
  struct tm timeinfo;
  if (!getLocalTime(&timeinfo)) {
    Serial.println("Failed to obtain time");
     digitalWrite(WIFI_Fail_LED_PIN, HIGH);
      digitalWrite(WIFI_OK_LED_PIN, LOW);
     
    return;
  }
  Serial.println(&timeinfo, "%A, %Y-%m-%d %H:%M:%S");
}


void wifiSetup() {
  WiFi.mode(WIFI_STA);
#if WIFI_SMARTCONFIG
  WiFi.begin();
#else
  WiFi.begin(WIFI_SSID, WIFI_PASS);
#endif

  for (int i = 0; ; i++) {
    Serial.println("Connecting to WiFi...");
    //digitalWrite(WIFI_Get_LED_PIN, HIGH);
    digitalWrite(WIFI_OK_LED_PIN, LOW);
    delay(500);
    digitalWrite(WIFI_Get_LED_PIN, HIGH);
    delay(500);
    digitalWrite(WIFI_Get_LED_PIN, LOW);
     
    if (WiFi.status() == WL_CONNECTED) {
      digitalWrite(WIFI_Fail_LED_PIN, LOW);
      digitalWrite(WIFI_Get_LED_PIN, LOW);
      digitalWrite(WIFI_OK_LED_PIN, HIGH);
      break;
    }
#if WIFI_SMARTCONFIG
  if(i > 30)
    break;
#endif    
  }

#if WIFI_SMARTCONFIG
  if (WiFi.status() != WL_CONNECTED) {
    WiFi.mode(WIFI_AP_STA);
    WiFi.beginSmartConfig();

    //Wait for SmartConfig packet from mobile
    Serial.println("Waiting for SmartConfig.");
    while (!WiFi.smartConfigDone()) {
      Serial.print(".");
      //pixels.setPixelColor(0, pixels.Color(0, 255, 0)); // red
     // pixels.show();
      delay(1000);
    }

    Serial.println("");
    Serial.println("SmartConfig received.");

    //Wait for WiFi to connect to AP
    Serial.println("Waiting for WiFi");
    while (WiFi.status() != WL_CONNECTED) {
      delay(1000);
      Serial.print(",");
      
    }
  }
  Serial.println("WiFi Connected.");
#endif

  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());
  Serial.print("IP Address: ");
  Serial.println(WiFi.localIP());
}
